@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 2002 Jasper Huijsmans (huysmans@users.sourceforge.net)
  *  Copyright (C) 2003 Benedikt Meurer (benedikt.meurer@unix-ag.uni-siegen.de)
- *  Copyright (c) 2004-2007 Brian Tarricone <bjt23@cornell.edu>
+ *  Copyright (c) 2004-2007 Brian Tarricone <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -45,10 +45,10 @@
 #endif
 
 #include <glib.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
+#include <libxfce4windowing/libxfce4windowing.h>
 
 #include "xfdesktop-common.h"
 #include "xfce-backdrop.h" /* for XfceBackdropImageStyle */
@@ -195,45 +195,34 @@ xfdesktop_remove_whitspaces(gchar* str)
 }
 
 
-static GtkWidget*
-create_menu_item(GtkWidget* label, GtkWidget* image)
+static GtkWidget *
+create_menu_item(const gchar *name,
+                 GtkWidget *image)
 {
     GtkWidget *mi;
-    GtkWidget *box;
-    GtkSettings *settings;
-    gboolean show_icons = TRUE;
 
-    /* create item */
-    mi = gtk_menu_item_new ();
-
-    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-    gtk_label_set_xalign (GTK_LABEL (label), 0.0);
-
-    box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-    gtk_widget_set_halign (label, GTK_ALIGN_START);
-
-    /* Add the image and label to the box, add the box to the menu item */
-    if (image && GTK_IS_WIDGET(image)) {
-        if (GTK_IS_IMAGE (image))
-            gtk_image_set_pixel_size (GTK_IMAGE (image), 16);
-
-        /* only add the widget if it exists */
-        gtk_widget_show (image);
-
-        gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
-    }
-
-    gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 6);
-    gtk_widget_show_all (box);
-    gtk_container_add (GTK_CONTAINER (mi), box);
-
-    if (gtk_widget_has_screen(label)) {
-        settings = gtk_settings_get_for_screen(gtk_widget_get_screen(label));
+    if (image == NULL) {
+        mi = gtk_menu_item_new_with_label(name);
     } else {
-        settings = gtk_settings_get_default();
+        GtkSettings *settings;
+        gboolean show_icons = TRUE;
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+        mi = gtk_image_menu_item_new_with_label(name);
+        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), image);
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+        if (gtk_widget_has_screen(image)) {
+            settings = gtk_settings_get_for_screen(gtk_widget_get_screen(image));
+        } else {
+            settings = gtk_settings_get_default();
+        }
+        gtk_widget_show(image);
+        g_object_get(settings, "gtk-menu-images", &show_icons, NULL);
+        gtk_widget_set_visible(image, show_icons);
     }
-    g_object_get(settings, "gtk-menu-images", &show_icons, NULL);
-    gtk_widget_set_visible(image, show_icons);
+
+    gtk_widget_show(mi);
 
     return mi;
 }
@@ -245,10 +234,10 @@ GtkWidget*
 xfdesktop_menu_create_menu_item_with_markup(const gchar *name,
                                             GtkWidget   *image)
 {
-    GtkWidget *label = gtk_label_new (NULL);
-
+    GtkWidget *mi = create_menu_item(name, image);
+    GtkWidget *label = gtk_bin_get_child(GTK_BIN(mi));
     gtk_label_set_markup(GTK_LABEL(label), name);
-    return create_menu_item(label, image);
+    return mi;
 }
 
 
@@ -257,9 +246,9 @@ GtkWidget*
 xfdesktop_menu_create_menu_item_with_mnemonic(const gchar *name,
                                               GtkWidget   *image)
 {
-    GtkWidget *label = gtk_label_new_with_mnemonic(name);
-
-    return create_menu_item(label, image);
+    GtkWidget *mi = create_menu_item(name, image);
+    gtk_menu_item_set_use_underline(GTK_MENU_ITEM(mi), TRUE);
+    return mi;
 }
 
 
@@ -332,6 +321,60 @@ xfdesktop_get_current_monitor_num(GdkDisplay *display)
     return xfdesktop_get_monitor_num(display, monitor);
 }
 
+/* Gets the workspace number across all workspace groups, and also returns
+ * the total number of workspaces.
+ */
+gboolean
+xfdesktop_workspace_get_number_and_total(XfwWorkspaceManager *workspace_manager,
+                                         XfwWorkspace *workspace,
+                                         gint *workspace_number,
+                                         gint *total_workspace_count)
+{
+    XfwWorkspaceGroup *group = xfw_workspace_get_workspace_group(workspace);
+
+    g_return_val_if_fail(workspace_number != NULL, FALSE);
+    g_return_val_if_fail(total_workspace_count != NULL, FALSE);
+
+    *workspace_number = -1;
+    *total_workspace_count = 0;
+
+    for (GList *l = xfw_workspace_manager_list_workspace_groups(workspace_manager);
+         l != NULL;
+         l = l->next)
+    {
+        XfwWorkspaceGroup *g = XFW_WORKSPACE_GROUP(l->data);
+        if (g == group) {
+            *workspace_number = *total_workspace_count + xfw_workspace_get_number(workspace);
+        }
+        *total_workspace_count += xfw_workspace_group_get_workspace_count(XFW_WORKSPACE_GROUP(l->data));
+    }
+
+    return *workspace_number >= 0;
+}
+
+GtkWindow *
+xfdesktop_find_toplevel(GtkWidget *widget)
+{
+    GtkWidget *toplevel = NULL;
+    GtkWidget *cur = widget;
+
+    while (cur != NULL && toplevel == NULL) {
+        if (GTK_IS_MENU(cur)) {
+            toplevel = gtk_menu_get_attach_widget(GTK_MENU(cur));
+            if (!GTK_IS_WINDOW(toplevel)) {
+                toplevel = NULL;
+            }
+        }
+
+        if (toplevel == NULL && GTK_IS_WINDOW(cur)) {
+            toplevel = cur;
+        }
+
+        cur = gtk_widget_get_parent(cur);
+    }
+
+    return toplevel != NULL && GTK_IS_WINDOW(toplevel) ? GTK_WINDOW(toplevel) : NULL;
+}
 
 
 void
