@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2004-2009 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2004-2009 Brian Tarricone, <brian@tarricone.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "xfdesktop-application.h"
 
@@ -50,8 +51,9 @@ main(int argc, char **argv)
     xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
 
     app = xfdesktop_application_get();
+    g_application_add_option_group(G_APPLICATION(app), xfce_sm_client_get_option_group(argc, argv));
 
-    ret = xfdesktop_application_run(app, argc, argv);
+    ret = g_application_run(G_APPLICATION(app), argc, argv);
 
     g_object_unref(app);
 
