@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright(c) 2006      Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright(c) 2006      Brian Tarricone, <brian@tarricone.org>
  *  Copyright(c) 2010-2011 Jannis Pohlmann, <jannis@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -68,8 +68,6 @@ typedef struct {
     GFile *file;
 } ExecuteData;
 
-static void xfdesktop_file_utils_add_emblems(GdkPixbuf *pix, GList *emblems);
-
 static XfdesktopTrash       *xfdesktop_file_utils_peek_trash_proxy(void);
 static XfdesktopFileManager *xfdesktop_file_utils_peek_filemanager_proxy(void);
 static XfdesktopFileManager1 *xfdesktop_file_utils_peek_filemanager_fdo_proxy(void);
@@ -403,219 +401,51 @@ xfdesktop_file_utils_file_list_free(GList *file_list)
 
 static GdkPixbuf *xfdesktop_fallback_icon = NULL;
 static gint xfdesktop_fallback_icon_size = -1;
+static gint xfdesktop_fallback_icon_scale = -1;
 
 GdkPixbuf *
-xfdesktop_file_utils_get_fallback_icon(gint size)
+xfdesktop_file_utils_get_fallback_icon(gint size,
+                                       gint scale)
 {
     g_return_val_if_fail(size > 0, NULL);
 
-    if(size != xfdesktop_fallback_icon_size && xfdesktop_fallback_icon) {
+    if((size != xfdesktop_fallback_icon_size || scale != xfdesktop_fallback_icon_scale) && xfdesktop_fallback_icon) {
         g_object_unref(G_OBJECT(xfdesktop_fallback_icon));
         xfdesktop_fallback_icon = NULL;
     }
 
     if(!xfdesktop_fallback_icon) {
         xfdesktop_fallback_icon = gdk_pixbuf_new_from_file_at_size(DATADIR "/pixmaps/xfdesktop/xfdesktop-fallback-icon.png",
-                                                                   size,
-                                                                   size,
+                                                                   size * scale,
+                                                                   size * scale,
                                                                    NULL);
     }
 
     if(G_UNLIKELY(!xfdesktop_fallback_icon)) {
         /* this is kinda crappy, but hopefully should never happen */
-        xfdesktop_fallback_icon = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-                                                           "image-missing",
-                                                           size,
-                                                           GTK_ICON_LOOKUP_USE_BUILTIN,
-                                                           NULL);
-        if(gdk_pixbuf_get_width(xfdesktop_fallback_icon) != size
-           || gdk_pixbuf_get_height(xfdesktop_fallback_icon) != size)
+        xfdesktop_fallback_icon = gtk_icon_theme_load_icon_for_scale(gtk_icon_theme_get_default(),
+                                                                     "image-missing",
+                                                                     size,
+                                                                     scale,
+                                                                     GTK_ICON_LOOKUP_USE_BUILTIN,
+                                                                     NULL);
+        if(gdk_pixbuf_get_width(xfdesktop_fallback_icon) != size * scale
+           || gdk_pixbuf_get_height(xfdesktop_fallback_icon) != size * scale)
         {
             GdkPixbuf *tmp = gdk_pixbuf_scale_simple(xfdesktop_fallback_icon,
-                                                     size, size,
-                                                     GDK_INTERP_BILINEAR);
+                                                     size * scale, size,
+                                                     GDK_INTERP_BILINEAR * scale);
             g_object_unref(G_OBJECT(xfdesktop_fallback_icon));
             xfdesktop_fallback_icon = tmp;
         }
     }
 
     xfdesktop_fallback_icon_size = size;
+    xfdesktop_fallback_icon_scale = scale;
 
     return GDK_PIXBUF(g_object_ref(G_OBJECT(xfdesktop_fallback_icon)));
 }
 
-GdkPixbuf *
-xfdesktop_file_utils_get_icon(GIcon *icon,
-                              gint width,
-                              gint height,
-                              guint opacity)
-{
-    GtkIconTheme *itheme = gtk_icon_theme_get_default();
-    GdkPixbuf *pix = NULL;
-    GIcon *base_icon = NULL;
-    gint size = MIN(width, height);
-
-    g_return_val_if_fail(width > 0 && height > 0 && icon != NULL, NULL);
-
-    /* Extract the base icon if available */
-    if(G_IS_EMBLEMED_ICON(icon))
-        base_icon = g_emblemed_icon_get_icon(G_EMBLEMED_ICON(icon));
-    else
-        base_icon = icon;
-
-    if(!base_icon)
-        return NULL;
-
-    if(G_IS_THEMED_ICON(base_icon)) {
-      GtkIconInfo *icon_info = gtk_icon_theme_lookup_by_gicon(itheme,
-                                                              base_icon, size,
-                                                              ITHEME_FLAGS);
-      if(icon_info) {
-          GdkPixbuf *pix_theme = gtk_icon_info_load_icon(icon_info, NULL);
-          // these icons are owned by GtkIconTheme and shouldn't be modified
-          pix = gdk_pixbuf_copy(pix_theme);
-          g_object_unref(pix_theme);
-          g_object_unref(icon_info);
-      }
-    } else if(G_IS_LOADABLE_ICON(base_icon)) {
-        GInputStream *stream = g_loadable_icon_load(G_LOADABLE_ICON(base_icon),
-                                                    size, NULL, NULL, NULL);
-        if(stream) {
-            pix = gdk_pixbuf_new_from_stream_at_scale(stream, width, height, TRUE, NULL, NULL);
-            g_object_unref(stream);
-        }
-    } else if(G_IS_FILE_ICON(base_icon)) {
-        GFile *file = g_file_icon_get_file(G_FILE_ICON(icon));
-        gchar *path = g_file_get_path(file);
-
-        pix = gdk_pixbuf_new_from_file_at_size(path, width, height, NULL);
-
-        g_free(path);
-        g_object_unref(file);
-    }
-
-    if (G_LIKELY(pix != NULL)) {
-        gint pix_width = gdk_pixbuf_get_width(pix);
-        gint pix_height = gdk_pixbuf_get_height(pix);
-
-        if (pix_width > width || pix_height > height) {
-            GdkPixbuf *scaled = exo_gdk_pixbuf_scale_down(pix, TRUE, width, height);
-            g_object_unref(pix);
-            pix = scaled;
-        }
-    } else {
-        pix = xfdesktop_file_utils_get_fallback_icon(size);
-        if (G_UNLIKELY(pix == NULL)) {
-            g_warning("Unable to find fallback icon");
-            return NULL;
-        }
-    }
-
-    /* Add the emblems */
-    if(G_IS_EMBLEMED_ICON(icon))
-        xfdesktop_file_utils_add_emblems(pix, g_emblemed_icon_get_emblems(G_EMBLEMED_ICON(icon)));
-
-    if(opacity != 100) {
-        GdkPixbuf *tmp = exo_gdk_pixbuf_lucent(pix, opacity);
-        g_object_unref(G_OBJECT(pix));
-        pix = tmp;
-    }
-
-    return pix;
-}
-
-static void
-xfdesktop_file_utils_add_emblems(GdkPixbuf *pix, GList *emblems)
-{
-    GdkPixbuf *emblem_pix = NULL;
-    gint max_emblems;
-    gint pix_width, pix_height;
-    gint emblem_size;
-    gint dest_x, dest_y, dest_width, dest_height;
-    gint position;
-    GList *iter;
-    GtkIconTheme *itheme = gtk_icon_theme_get_default();
-
-    g_return_if_fail(pix != NULL);
-
-    pix_width = gdk_pixbuf_get_width(pix);
-    pix_height = gdk_pixbuf_get_height(pix);
-
-    emblem_size = MIN(pix_width, pix_height) / 2;
-
-    /* render up to four emblems for sizes from 48 onwards, else up to 2 emblems */
-    max_emblems = (pix_height < 48 && pix_width < 48) ? 2 : 4;
-
-    for(iter = g_list_last(emblems), position = 0;
-        iter != NULL && position < max_emblems; iter = iter->prev) {
-        /* extract the icon from the emblem and load it */
-        GIcon *emblem = g_emblem_get_icon(iter->data);
-        GtkIconInfo *icon_info = gtk_icon_theme_lookup_by_gicon(itheme,
-                                                                emblem,
-                                                                emblem_size,
-                                                                ITHEME_FLAGS);
-        if(icon_info) {
-            emblem_pix = gtk_icon_info_load_icon(icon_info, NULL);
-            g_object_unref(icon_info);
-        }
-
-        if(emblem_pix) {
-            if(gdk_pixbuf_get_width(emblem_pix) != emblem_size
-               || gdk_pixbuf_get_height(emblem_pix) != emblem_size)
-            {
-                GdkPixbuf *tmp = gdk_pixbuf_scale_simple(emblem_pix,
-                                                         emblem_size,
-                                                         emblem_size,
-                                                         GDK_INTERP_BILINEAR);
-                g_object_unref(emblem_pix);
-                emblem_pix = tmp;
-            }
-
-            dest_width = pix_width - emblem_size;
-            dest_height = pix_height - emblem_size;
-
-            switch(position) {
-                case 0: /* bottom right */
-                    dest_x = dest_width;
-                    dest_y = dest_height;
-                    break;
-                case 1: /* bottom left */
-                    dest_x = 0;
-                    dest_y = dest_height;
-                    break;
-                case 2: /* upper left */
-                    dest_x = dest_y = 0;
-                    break;
-                case 3: /* upper right */
-                    dest_x = dest_width;
-                    dest_y = 0;
-                    break;
-                default:
-                    g_warning("Invalid emblem position in xfdesktop_file_utils_add_emblems");
-            }
-
-            DBG("calling gdk_pixbuf_composite(%p, %p, %d, %d, %d, %d, %d, %d, %.1f, %.1f, %d, %d) pixbuf w: %d h: %d",
-                emblem_pix, pix,
-                dest_x, dest_y,
-                emblem_size, emblem_size,
-                dest_x, dest_y,
-                1.0, 1.0, GDK_INTERP_BILINEAR, 255, pix_width, pix_height);
-
-            /* Add the emblem */
-            gdk_pixbuf_composite(emblem_pix, pix,
-                                 dest_x, dest_y,
-                                 emblem_size, emblem_size,
-                                 dest_x, dest_y,
-                                 1.0, 1.0, GDK_INTERP_BILINEAR, 255);
-
-            g_object_unref(emblem_pix);
-            emblem_pix = NULL;
-
-            position++;
-        }
-    }
-}
-
 void
 xfdesktop_file_utils_set_window_cursor(GtkWindow *window,
                                        GdkCursorType cursor_type)
@@ -717,40 +547,119 @@ xfdesktop_file_utils_app_info_launch(GAppInfo *app_info,
     return result;
 }
 
-void
-xfdesktop_file_utils_open_folder(GFile *file,
-                                 GdkScreen *screen,
-                                 GtkWindow *parent)
+static void
+report_open_folders_error(GtkWindow *parent,
+                          GError *error)
 {
-    gchar *uri = NULL;
+    xfce_message_dialog(parent,
+                        _("Launch Error"), "dialog-error",
+                        _("The folder could not be opened"),
+                        error->message,
+                        XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
+                        NULL);
+}
+
+static gboolean
+xfdesktop_file_utils_open_folders_fallback(const gchar *const *uris,
+                                           GdkScreen *screen,
+                                           GtkWindow *parent,
+                                           gboolean report_errors)
+{
+    gboolean succeeded = TRUE;
+
+    for (gint i = 0; uris[i] != NULL; ++i) {
+        GError *error = NULL;
+
+        if (!exo_execute_preferred_application_on_screen("FileManager",
+                                                         uris[i],
+                                                         NULL,
+                                                         NULL,
+                                                         screen,
+                                                         &error))
+        {
+            if (report_errors) {
+                report_open_folders_error(parent, error);
+            }
+
+            g_clear_error(&error);
+            succeeded = FALSE;
+        }
+    }
+
+    return succeeded;
+}
+
+typedef struct {
+    gchar **uris;
+    GdkScreen *screen;
+    GtkWindow *parent;
+} ShowFoldersFallbackData;
+
+static void
+show_folders_finished(GObject *source_object,
+                      GAsyncResult *res,
+                      gpointer user_data)
+{
+    ShowFoldersFallbackData *data = user_data;
     GError *error = NULL;
 
-    g_return_if_fail(G_IS_FILE(file));
+    if (!xfdesktop_file_manager1_call_show_folders_finish(XFDESKTOP_FILE_MANAGER1(source_object), res, &error)) {
+        if (!xfdesktop_file_utils_open_folders_fallback((const gchar *const *)data->uris,
+                                                        data->screen,
+                                                        data->parent,
+                                                        FALSE))
+        {
+            report_open_folders_error(data->parent, error);
+        }
+
+        g_clear_error(&error);
+    }
+
+    g_strfreev(data->uris);
+    if (data->parent != NULL) {
+        g_object_unref(data->parent);
+    }
+    g_slice_free(ShowFoldersFallbackData, data);
+}
+
+void
+xfdesktop_file_utils_open_folders(GList *files,
+                                  GdkScreen *screen,
+                                  GtkWindow *parent)
+{
+    XfdesktopFileManager1 *fileman_fdo_proxy;
+    gchar **uris;
+
+    g_return_if_fail(files != NULL);
     g_return_if_fail(GDK_IS_SCREEN(screen) || GTK_IS_WINDOW(parent));
 
     if(!screen)
         screen = gtk_widget_get_screen(GTK_WIDGET(parent));
 
-    uri = g_file_get_uri(file);
+    uris = xfdesktop_file_utils_file_list_to_uri_array(files);
+    g_return_if_fail(uris != NULL && uris[0] != NULL);
 
-    if(!exo_execute_preferred_application_on_screen("FileManager",
-                                                    uri,
-                                                    NULL,
-                                                    NULL,
-                                                    screen,
-                                                    &error))
-    {
-        xfce_message_dialog(parent,
-                            _("Launch Error"), "dialog-error",
-                            _("The folder could not be opened"),
-                            error->message,
-                            XFCE_BUTTON_TYPE_MIXED, "window-close", _("_Close"), GTK_RESPONSE_ACCEPT,
-                            NULL);
+    fileman_fdo_proxy = xfdesktop_file_utils_peek_filemanager_fdo_proxy();
+    if (fileman_fdo_proxy != NULL) {
+        ShowFoldersFallbackData *data = g_slice_new0(ShowFoldersFallbackData);
+        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
 
-        g_clear_error(&error);
+        data->uris = uris;
+        data->screen = screen;
+        data->parent = parent != NULL ? g_object_ref(parent) : NULL;
+
+        xfdesktop_file_manager1_call_show_folders(fileman_fdo_proxy,
+                                                  (const gchar *const *)uris,
+                                                  startup_id,
+                                                  NULL,
+                                                  show_folders_finished,
+                                                  data);
+
+        g_free(startup_id);
+    } else {
+        xfdesktop_file_utils_open_folders_fallback((const gchar *const *)uris,screen, parent, TRUE);
+        g_strfreev(uris);
     }
-
-    g_free(uri);
 }
 
 static void
