@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright(c) 2006      Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright(c) 2006      Brian Tarricone, <brian@tarricone.org>
  *  Copyright(c) 2010-2011 Jannis Pohlmann, <jannis@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -47,12 +47,8 @@ gchar *xfdesktop_file_utils_file_list_to_string(GList *file_list);
 gchar **xfdesktop_file_utils_file_list_to_uri_array(GList *file_list);
 void xfdesktop_file_utils_file_list_free(GList *file_list);
 
-GdkPixbuf *xfdesktop_file_utils_get_fallback_icon(gint size);
-
-GdkPixbuf *xfdesktop_file_utils_get_icon(GIcon *icon,
-                                         gint width,
-                                         gint height,
-                                         guint opacity);
+GdkPixbuf *xfdesktop_file_utils_get_fallback_icon(gint size,
+                                                  gint scale);
 
 void xfdesktop_file_utils_set_window_cursor(GtkWindow *window,
                                             GdkCursorType cursor_type);
@@ -63,9 +59,9 @@ gboolean xfdesktop_file_utils_app_info_launch(GAppInfo *app_info,
                                               GAppLaunchContext *context,
                                               GError **error);
 
-void xfdesktop_file_utils_open_folder(GFile *file,
-                                      GdkScreen *screen,
-                                      GtkWindow *parent);
+void xfdesktop_file_utils_open_folders(GList *files,
+                                       GdkScreen *screen,
+                                       GtkWindow *parent);
 void xfdesktop_file_utils_rename_file(GFile *file,
                                       GdkScreen *screen,
                                       GtkWindow *parent);
