@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright(c) 2006 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright(c) 2006 Brian Tarricone, <brian@tarricone.org>
  *  Copyright(c) 2006 Benedikt Meurer, <benny@xfce.org>
  *  Copyright(c) 2010-2011 Jannis Pohlmann, <jannis@xfce.org>
  *
@@ -62,7 +62,6 @@ struct _XfdesktopRegularFileIconPrivate
 {
     gchar *display_name;
     gchar *tooltip;
-    guint pix_opacity;
     GFileInfo *file_info;
     GFileInfo *filesystem_info;
     GFile *file;
@@ -78,12 +77,8 @@ static void xfdesktop_regular_file_icon_finalize(GObject *obj);
 static void xfdesktop_regular_file_icon_set_thumbnail_file(XfdesktopIcon *icon, GFile *file);
 static void xfdesktop_regular_file_icon_delete_thumbnail_file(XfdesktopIcon *icon);
 
-static GdkPixbuf *xfdesktop_regular_file_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                                          gint width, gint height);
 static const gchar *xfdesktop_regular_file_icon_peek_label(XfdesktopIcon *icon);
 static gchar *xfdesktop_regular_file_icon_get_identifier(XfdesktopIcon *icon);
-static GdkPixbuf *xfdesktop_regular_file_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
-                                                                  gint width, gint height);
 static const gchar *xfdesktop_regular_file_icon_peek_tooltip(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_regular_file_icon_get_allowed_drag_actions(XfdesktopIcon *icon);
 static GdkDragAction xfdesktop_regular_file_icon_get_allowed_drop_actions(XfdesktopIcon *icon,
@@ -92,6 +87,7 @@ static gboolean xfdesktop_regular_file_icon_do_drop_dest(XfdesktopIcon *icon,
                                                          GList *src_icons,
                                                          GdkDragAction action);
 
+static GIcon *xfdesktop_regular_file_icon_get_gicon(XfdesktopFileIcon *icon);
 static GFileInfo *xfdesktop_regular_file_icon_peek_file_info(XfdesktopFileIcon *icon);
 static GFileInfo *xfdesktop_regular_file_icon_peek_filesystem_info(XfdesktopFileIcon *icon);
 static GFile *xfdesktop_regular_file_icon_peek_file(XfdesktopFileIcon *icon);
@@ -99,6 +95,8 @@ static void xfdesktop_regular_file_icon_update_file_info(XfdesktopFileIcon *icon
                                                          GFileInfo *info);
 static gboolean xfdesktop_regular_file_can_write_parent(XfdesktopFileIcon *icon);
 
+static gboolean is_folder_icon(GFile *file);
+
 #ifdef HAVE_THUNARX
 static void xfdesktop_regular_file_icon_tfi_init(ThunarxFileInfoIface *iface);
 
@@ -114,6 +112,29 @@ G_DEFINE_TYPE_WITH_PRIVATE(XfdesktopRegularFileIcon, xfdesktop_regular_file_icon
 #endif
 
 
+/* So much for standards */
+static const gchar *folder_icon_names[] = {
+    "Folder.jpg",
+    "folder.jpg",
+    "Folder.JPG",
+    "folder.JPG",
+    "folder.jpeg",
+    "folder.JPEG",
+    "Folder.JPEG",
+    "Folder.jpeg",
+    "Cover.jpg",
+    "cover.jpg",
+    "Cover.jpeg",
+    "cover.jpeg",
+    "albumart.jpg",
+    "albumart.jpeg",
+    "fanart.jpg",
+    "Fanart.jpg",
+    "fanart.JPG",
+    "Fanart.JPG",
+    "FANART.JPG",
+    "FANART.jpg",
+};
 
 static void
 xfdesktop_regular_file_icon_class_init(XfdesktopRegularFileIconClass *klass)
@@ -124,10 +145,8 @@ xfdesktop_regular_file_icon_class_init(XfdesktopRegularFileIconClass *klass)
 
     gobject_class->finalize = xfdesktop_regular_file_icon_finalize;
 
-    icon_class->peek_pixbuf = xfdesktop_regular_file_icon_peek_pixbuf;
     icon_class->peek_label = xfdesktop_regular_file_icon_peek_label;
     icon_class->get_identifier = xfdesktop_regular_file_icon_get_identifier;
-    icon_class->peek_tooltip_pixbuf = xfdesktop_regular_file_icon_peek_tooltip_pixbuf;
     icon_class->peek_tooltip = xfdesktop_regular_file_icon_peek_tooltip;
     icon_class->get_allowed_drag_actions = xfdesktop_regular_file_icon_get_allowed_drag_actions;
     icon_class->get_allowed_drop_actions = xfdesktop_regular_file_icon_get_allowed_drop_actions;
@@ -135,6 +154,7 @@ xfdesktop_regular_file_icon_class_init(XfdesktopRegularFileIconClass *klass)
     icon_class->set_thumbnail_file = xfdesktop_regular_file_icon_set_thumbnail_file;
     icon_class->delete_thumbnail_file = xfdesktop_regular_file_icon_delete_thumbnail_file;
 
+    file_icon_class->get_gicon = xfdesktop_regular_file_icon_get_gicon;
     file_icon_class->peek_file_info = xfdesktop_regular_file_icon_peek_file_info;
     file_icon_class->peek_filesystem_info = xfdesktop_regular_file_icon_peek_filesystem_info;
     file_icon_class->peek_file = xfdesktop_regular_file_icon_peek_file;
@@ -147,7 +167,6 @@ static void
 xfdesktop_regular_file_icon_init(XfdesktopRegularFileIcon *icon)
 {
     icon->priv = xfdesktop_regular_file_icon_get_instance_private(icon);
-    icon->priv->pix_opacity = 100;
     icon->priv->display_name = NULL;
 }
 
@@ -155,11 +174,6 @@ static void
 xfdesktop_regular_file_icon_finalize(GObject *obj)
 {
     XfdesktopRegularFileIcon *icon = XFDESKTOP_REGULAR_FILE_ICON(obj);
-    GtkIconTheme *itheme = gtk_icon_theme_get_for_screen(icon->priv->gscreen);
-
-    g_signal_handlers_disconnect_by_func(G_OBJECT(itheme),
-                                         G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
-                                         icon);
 
     if(icon->priv->file_info)
         g_object_unref(icon->priv->file_info);
@@ -217,7 +231,6 @@ xfdesktop_regular_file_icon_delete_thumbnail_file(XfdesktopIcon *icon)
 
     xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(icon));
 
-    xfdesktop_icon_invalidate_pixbuf(icon);
     xfdesktop_icon_pixbuf_changed(icon);
 }
 
@@ -238,7 +251,6 @@ xfdesktop_regular_file_icon_set_thumbnail_file(XfdesktopIcon *icon, GFile *file)
 
     xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(icon));
 
-    xfdesktop_icon_invalidate_pixbuf(icon);
     xfdesktop_icon_pixbuf_changed(icon);
 }
 
@@ -264,7 +276,6 @@ cb_show_thumbnails_notify(GObject *gobject,
         XF_DEBUG("show-thumbnails changed! now: %s", show_thumbnails ? "TRUE" : "FALSE");
         regular_file_icon->priv->show_thumbnails = show_thumbnails;
         xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(regular_file_icon));
-        xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(regular_file_icon));
         xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(regular_file_icon));
     }
 }
@@ -285,72 +296,39 @@ xfdesktop_check_file_is_valid(const gchar *folder, const gchar *file)
     return path;
 }
 
+static gboolean
+is_folder_icon(GFile *file) {
+    if (file != NULL) {
+        gboolean match = FALSE;
+
+        gchar *filename = g_file_get_basename(file);
+        for (gsize i = 0; i < G_N_ELEMENTS(folder_icon_names); ++i) {
+            if (g_strcmp0(filename, folder_icon_names[i]) == 0) {
+                match = TRUE;
+                break;
+            }
+        }
+
+        g_free(filename);
+        return match;
+    } else {
+        return FALSE;
+    }
+}
+
 static gchar *
 xfdesktop_load_icon_location_from_folder(XfdesktopFileIcon *icon)
 {
     gchar *icon_file = g_file_get_path(xfdesktop_file_icon_peek_file(icon));
-    gchar *path;
+    gchar *path = NULL;
 
     g_return_val_if_fail(icon_file, NULL);
 
-    /* So much for standards */
-    path = xfdesktop_check_file_is_valid(icon_file, "Folder.jpg");
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "folder.jpg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "Folder.JPG");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "folder.JPG");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "folder.jpeg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "folder.JPEG");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "Folder.JPEG");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "Folder.jpeg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "Cover.jpg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "cover.jpg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "Cover.jpeg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "cover.jpeg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "albumart.jpg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "albumart.jpeg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "fanart.jpg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "Fanart.jpg");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "fanart.JPG");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "Fanart.JPG");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "FANART.JPG");
-    }
-    if(path == NULL) {
-        path = xfdesktop_check_file_is_valid(icon_file, "FANART.jpg");
+    for (gsize i = 0; i < G_N_ELEMENTS(folder_icon_names); ++i) {
+        path = xfdesktop_check_file_is_valid(icon_file, folder_icon_names[i]);
+        if (path != NULL) {
+            break;
+        }
     }
 
     g_free(icon_file);
@@ -472,17 +450,18 @@ xfdesktop_load_icon_from_desktop_file(XfdesktopRegularFileIcon *regular_icon)
 }
 
 static GIcon *
-xfdesktop_regular_file_icon_load_icon(XfdesktopIcon *icon)
+xfdesktop_regular_file_icon_get_gicon(XfdesktopFileIcon *icon)
 {
     XfdesktopRegularFileIcon *regular_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
     XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(icon);
+    GIcon *base_gicon = NULL;
     GIcon *gicon = NULL;
 
     TRACE("entering");
 
     /* Try to load the icon referenced in the .desktop file */
     if(xfdesktop_file_utils_is_desktop_file(regular_icon->priv->file_info)) {
-        gicon = xfdesktop_load_icon_from_desktop_file(regular_icon);
+        base_gicon = xfdesktop_load_icon_from_desktop_file(regular_icon);
 
     } else if(g_file_info_get_file_type(regular_icon->priv->file_info) == G_FILE_TYPE_DIRECTORY) {
         /* Try to load a thumbnail from the standard folder image locations */
@@ -494,7 +473,7 @@ xfdesktop_regular_file_icon_load_icon(XfdesktopIcon *icon)
         if(thumbnail_file) {
             /* If there's a folder thumbnail, use it */
             regular_icon->priv->thumbnail_file = g_file_new_for_path(thumbnail_file);
-            gicon = g_file_icon_new(regular_icon->priv->thumbnail_file);
+            base_gicon = g_file_icon_new(regular_icon->priv->thumbnail_file);
             g_free(thumbnail_file);
         }
 
@@ -506,9 +485,9 @@ xfdesktop_regular_file_icon_load_icon(XfdesktopIcon *icon)
 
             /* Don't use thumbnails for svg, use the file itself */
             if(g_strcmp0(mimetype, "image/svg+xml") == 0)
-                gicon = g_file_icon_new(regular_icon->priv->file);
+                base_gicon = g_file_icon_new(regular_icon->priv->file);
             else
-                gicon = g_file_icon_new(regular_icon->priv->thumbnail_file);
+                base_gicon = g_file_icon_new(regular_icon->priv->thumbnail_file);
 
             g_free(mimetype);
             g_free(file);
@@ -516,16 +495,15 @@ xfdesktop_regular_file_icon_load_icon(XfdesktopIcon *icon)
     }
 
     /* If we still don't have an icon, use the default */
-    if(!G_IS_ICON(gicon)) {
-        gicon = g_file_info_get_icon(regular_icon->priv->file_info);
-        if(G_IS_ICON(gicon))
-            g_object_ref(gicon);
+    if(!G_IS_ICON(base_gicon)) {
+        base_gicon = g_file_info_get_icon(regular_icon->priv->file_info);
+        if(G_IS_ICON(base_gicon))
+            g_object_ref(base_gicon);
     }
 
-    g_object_set(file_icon, "gicon", gicon, NULL);
-
     /* Add any user set emblems */
-    gicon = xfdesktop_file_icon_add_emblems(file_icon);
+    gicon = xfdesktop_file_icon_add_emblems(file_icon, base_gicon);
+    g_object_unref(base_gicon);
 
     /* load the unreadable emblem if necessary */
     if(!g_file_info_get_attribute_boolean(regular_icon->priv->file_info,
@@ -568,42 +546,6 @@ xfdesktop_regular_file_icon_load_icon(XfdesktopIcon *icon)
     return gicon;
 }
 
-static GdkPixbuf *
-xfdesktop_regular_file_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                        gint width, gint height)
-{
-    XfdesktopRegularFileIcon *regular_icon = XFDESKTOP_REGULAR_FILE_ICON(icon);
-    GIcon *gicon = NULL;
-    GdkPixbuf *pix = NULL;
-
-    if(!xfdesktop_file_icon_has_gicon(XFDESKTOP_FILE_ICON(icon)))
-        gicon = xfdesktop_regular_file_icon_load_icon(icon);
-    else
-        g_object_get(XFDESKTOP_FILE_ICON(icon), "gicon", &gicon, NULL);
-
-    pix = xfdesktop_file_utils_get_icon(gicon, width, height,
-                                        regular_icon->priv->pix_opacity);
-
-    return pix;
-}
-
-static GdkPixbuf *
-xfdesktop_regular_file_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
-                                                gint width, gint height)
-{
-    GIcon *gicon = NULL;
-    GdkPixbuf *tooltip_pix = NULL;
-
-    if(!xfdesktop_file_icon_has_gicon(XFDESKTOP_FILE_ICON(icon)))
-        gicon = xfdesktop_regular_file_icon_load_icon(icon);
-    else
-        g_object_get(XFDESKTOP_FILE_ICON(icon), "gicon", &gicon, NULL);
-
-    tooltip_pix = xfdesktop_file_utils_get_icon(gicon, width, height, 100);
-
-    return tooltip_pix;
-}
-
 static const gchar *
 xfdesktop_regular_file_icon_peek_label(XfdesktopIcon *icon)
 {
@@ -806,6 +748,7 @@ xfdesktop_regular_file_icon_peek_tooltip(XfdesktopIcon *icon)
             if(rcfile) {
                 xfce_rc_set_group(rcfile, "Desktop Entry");
                 comment = xfce_rc_read_entry(rcfile, "Comment", NULL);
+                xfce_rc_close(rcfile);
             }
             /* Prepend the comment to the tooltip */
             if(comment != NULL && *comment != '\0') {
@@ -815,8 +758,6 @@ xfdesktop_regular_file_icon_peek_tooltip(XfdesktopIcon *icon)
                                                                    tooltip);
                 g_free(tooltip);
             }
-
-            xfce_rc_close(rcfile);
         }
 
         g_free(time_string);
@@ -930,7 +871,6 @@ xfdesktop_regular_file_icon_update_file_info(XfdesktopFileIcon *icon,
 
     /* not really easy to check if this changed or not, so just invalidate it */
     xfdesktop_file_icon_invalidate_icon(XFDESKTOP_FILE_ICON(icon));
-    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(icon));
     xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
 }
 
@@ -942,7 +882,6 @@ cb_folder_contents_changed(GFileMonitor     *monitor,
                            gpointer          user_data)
 {
     XfdesktopRegularFileIcon *regular_file_icon;
-    gchar *thumbnail_file = NULL;
 
     if(!user_data || !XFDESKTOP_IS_REGULAR_FILE_ICON(user_data))
         return;
@@ -953,24 +892,36 @@ cb_folder_contents_changed(GFileMonitor     *monitor,
     if(!regular_file_icon->priv->show_thumbnails)
         return;
 
-    /* Already has a thumbnail */
-    if(regular_file_icon->priv->thumbnail_file != NULL)
-        return;
-
+    gboolean reload_icon;
     switch(event) {
         case G_FILE_MONITOR_EVENT_CREATED:
-                thumbnail_file = xfdesktop_load_icon_location_from_folder(XFDESKTOP_FILE_ICON(regular_file_icon));
-                if(thumbnail_file) {
-                    GFile *thumbnail = g_file_new_for_path(thumbnail_file);
-                    /* found a thumbnail file, apply it */
-                    xfdesktop_regular_file_icon_set_thumbnail_file(XFDESKTOP_ICON(regular_file_icon),
-                                                                   thumbnail);
-                    g_free(thumbnail_file);
-                }
+        case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+        case G_FILE_MONITOR_EVENT_DELETED:
+            reload_icon = is_folder_icon(file);
             break;
+
+        case G_FILE_MONITOR_EVENT_MOVED:
+        case G_FILE_MONITOR_EVENT_MOVED_IN:
+        case G_FILE_MONITOR_EVENT_MOVED_OUT:
+            reload_icon = is_folder_icon(file) || is_folder_icon(other_file);
+            break;
+
         default:
+            reload_icon = FALSE;
             break;
     }
+
+    if (reload_icon) {
+        gchar *thumbnail_file = xfdesktop_load_icon_location_from_folder(XFDESKTOP_FILE_ICON(regular_file_icon));
+        if (thumbnail_file) {
+            GFile *thumbnail = g_file_new_for_path(thumbnail_file);
+            /* found a thumbnail file, apply it */
+            xfdesktop_regular_file_icon_set_thumbnail_file(XFDESKTOP_ICON(regular_file_icon), thumbnail);
+            g_free(thumbnail_file);
+        } else {
+            xfdesktop_regular_file_icon_set_thumbnail_file(XFDESKTOP_ICON(regular_file_icon), NULL);
+        }
+    }
 }
 
 /* public API */
@@ -1005,14 +956,9 @@ xfdesktop_regular_file_icon_new(GFile *file,
 
     regular_file_icon->priv->fmanager = fmanager;
 
-    g_signal_connect_swapped(G_OBJECT(gtk_icon_theme_get_for_screen(screen)),
-                             "changed",
-                             G_CALLBACK(xfdesktop_icon_invalidate_pixbuf),
-                             regular_file_icon);
-
     if(g_file_info_get_file_type(regular_file_icon->priv->file_info) == G_FILE_TYPE_DIRECTORY) {
         regular_file_icon->priv->monitor = g_file_monitor(regular_file_icon->priv->file,
-                                                          G_FILE_MONITOR_NONE,
+                                                          G_FILE_MONITOR_WATCH_MOVES,
                                                           NULL,
                                                           NULL);
 
@@ -1030,18 +976,3 @@ xfdesktop_regular_file_icon_new(GFile *file,
     }
     return regular_file_icon;
 }
-
-void
-xfdesktop_regular_file_icon_set_pixbuf_opacity(XfdesktopRegularFileIcon *icon,
-                                       guint opacity)
-{
-    g_return_if_fail(XFDESKTOP_IS_REGULAR_FILE_ICON(icon) && opacity <= 100);
-
-    if(opacity == icon->priv->pix_opacity)
-        return;
-
-    icon->priv->pix_opacity = opacity;
-
-    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(icon));
-    xfdesktop_icon_pixbuf_changed(XFDESKTOP_ICON(icon));
-}
