@@ -90,6 +90,7 @@ static gboolean xfdesktop_volume_icon_populate_context_menu(XfdesktopIcon *icon,
                                                             GtkWidget *menu);
 
 static GIcon * xfdesktop_volume_icon_get_gicon(XfdesktopFileIcon *icon);
+static gdouble xfdesktop_volume_icon_get_opacity(XfdesktopFileIcon *icon);
 static GFileInfo *xfdesktop_volume_icon_peek_file_info(XfdesktopFileIcon *icon);
 static GFileInfo *xfdesktop_volume_icon_peek_filesystem_info(XfdesktopFileIcon *icon);
 static GFile *xfdesktop_volume_icon_peek_file(XfdesktopFileIcon *icon);
@@ -161,6 +162,7 @@ xfdesktop_volume_icon_class_init(XfdesktopVolumeIconClass *klass)
     icon_class->activate = xfdesktop_volume_icon_activate;
 
     file_icon_class->get_gicon = xfdesktop_volume_icon_get_gicon;
+    file_icon_class->get_icon_opacity = xfdesktop_volume_icon_get_opacity;
     file_icon_class->peek_file_info = xfdesktop_volume_icon_peek_file_info;
     file_icon_class->peek_filesystem_info = xfdesktop_volume_icon_peek_filesystem_info;
     file_icon_class->peek_file = xfdesktop_volume_icon_peek_file;
@@ -277,6 +279,11 @@ xfdesktop_volume_icon_get_gicon(XfdesktopFileIcon *icon)
     return gicon;
 }
 
+static gdouble
+xfdesktop_volume_icon_get_opacity(XfdesktopFileIcon *icon) {
+    return xfdesktop_volume_icon_is_mounted(XFDESKTOP_ICON(icon)) ? 1.0 : 0.5;
+}
+
 const gchar *
 xfdesktop_volume_icon_peek_label(XfdesktopIcon *icon)
 {
@@ -725,22 +732,15 @@ xfdesktop_volume_icon_menu_eject(GtkWidget *widget,
                                  gpointer user_data)
 {
     XfdesktopVolumeIcon *icon = XFDESKTOP_VOLUME_ICON(user_data);
-    GtkWindow *toplevel = xfdesktop_find_toplevel(widget);
-    GVolume *volume;
-    GMount *mount;
-    GMountOperation *operation = NULL;
+    GVolume *volume = xfdesktop_volume_icon_peek_volume(icon);
 
-    volume = xfdesktop_volume_icon_peek_volume(icon);
-    mount = g_volume_get_mount(volume);
-
-    if(!mount)
-        return;
-
-    if(g_volume_can_eject(volume)) {
+    if (volume != NULL && g_volume_can_eject(volume)) {
 #ifdef HAVE_LIBNOTIFY
         xfdesktop_notify_eject(volume);
 #endif
-        operation = gtk_mount_operation_new(toplevel);
+
+        GtkWindow *toplevel = xfdesktop_find_toplevel(widget);
+        GMountOperation *operation = gtk_mount_operation_new(toplevel);
         gtk_mount_operation_set_screen(GTK_MOUNT_OPERATION(operation),
                                        icon->priv->gscreen);
 
@@ -750,14 +750,11 @@ xfdesktop_volume_icon_menu_eject(GtkWidget *widget,
                                       NULL,
                                       xfdesktop_volume_icon_eject_finish,
                                       g_object_ref(icon));
+        g_object_unref(operation);
     } else {
         /* If we can't eject the volume try to unmount it */
         xfdesktop_volume_icon_menu_unmount(widget, user_data);
     }
-
-    g_object_unref(mount);
-    if(operation != NULL)
-        g_object_unref(operation);
 }
 
 static void
