@@ -169,6 +169,7 @@ xfdesktop_window_icon_manager_constructed(GObject *object)
     GtkTargetList *text_target_list;
     GtkTargetEntry *text_targets;
     gint n_text_targets = 0;
+    GList *groups;
 
     DBG("entering");
 
@@ -185,6 +186,7 @@ xfdesktop_window_icon_manager_constructed(GObject *object)
     wmanager->priv->icon_view = g_object_new(XFDESKTOP_TYPE_ICON_VIEW,
                                              "channel", xfdesktop_icon_view_manager_get_channel(XFDESKTOP_ICON_VIEW_MANAGER(wmanager)),
                                              "pixbuf-column", XFDESKTOP_ICON_VIEW_MODEL_COLUMN_IMAGE,
+                                             "icon-opacity-column", XFDESKTOP_ICON_VIEW_MODEL_COLUMN_IMAGE_OPACITY,
                                              "text-column", XFDESKTOP_ICON_VIEW_MODEL_COLUMN_LABEL,
                                              "search-column", XFDESKTOP_ICON_VIEW_MODEL_COLUMN_LABEL,
                                              "tooltip-icon-column", XFDESKTOP_ICON_VIEW_MODEL_COLUMN_TOOLTIP_IMAGE,
@@ -252,12 +254,11 @@ xfdesktop_window_icon_manager_constructed(GObject *object)
         workspace_created_cb(wmanager->priv->workspace_manager, XFW_WORKSPACE(l->data), wmanager);
     }
 
-    for (GList *l = g_list_reverse(xfw_workspace_manager_list_workspace_groups(wmanager->priv->workspace_manager));
-         l != NULL;
-         l = l->next)
-    {
+    groups = g_list_reverse(g_list_copy(xfw_workspace_manager_list_workspace_groups(wmanager->priv->workspace_manager)));
+    for (GList *l = groups; l != NULL; l = l->next) {
         workspace_changed_cb(XFW_WORKSPACE_GROUP(l->data), NULL, wmanager);
     }
+    g_list_free(groups);
 
     xfdesktop_window_icon_manager_populate_workspaces(wmanager);
 }
@@ -749,14 +750,14 @@ static void
 xfdesktop_window_icon_manager_workarea_changed(XfdesktopWindowIconManager *wmanager)
 {
     GdkRectangle workarea;
-
     xfdesktop_icon_view_manager_get_workarea(XFDESKTOP_ICON_VIEW_MANAGER(wmanager), &workarea);
-    DBG("moving icon view to %dx%d+%d+%d", workarea.width, workarea.height, workarea.x, workarea.y);
-    gtk_widget_set_size_request(GTK_WIDGET(wmanager->priv->icon_view), workarea.width, workarea.height);
-    gtk_fixed_move(xfdesktop_icon_view_manager_get_container(XFDESKTOP_ICON_VIEW_MANAGER(wmanager)),
-                   GTK_WIDGET(wmanager->priv->icon_view),
-                   workarea.x,
-                   workarea.y);
+
+    if (workarea.width > 0 && workarea.height > 0) {
+        gtk_widget_set_size_request(GTK_WIDGET(wmanager->priv->icon_view), workarea.width, workarea.height);
+    }
+
+    GtkFixed *container = xfdesktop_icon_view_manager_get_container(XFDESKTOP_ICON_VIEW_MANAGER(wmanager));
+    gtk_fixed_move(container, GTK_WIDGET(wmanager->priv->icon_view), workarea.x, workarea.y);
 }
 
 static void
